const Clock = require('common/clock.js');
const BCD = require('utils/bcd.js');


function ChangeICBit(cia, v, pos){
  let mask = ~(1 << pos);
  v = ((v === true || v === 1) ? 1 : 0) << pos;
  cia.__IC = (cia.__IC & mask) | v;
}


function TODTick(cia){
  cia.__TODTick += 1;
  if (cia.__TODTick === 6){
    cia.__TODTick = cia.__CTA & 0x01;
    if (cia.__10ths == 0x09){
      cia.__10ths = 0;
      if (cia.__sec == 0x59){
        cia.__sec = 0;
        if (cia.__min == 0x59){
          cia.__min = 0;
          if (cia.__hour == 0x11 || cia.__hour == 0x91){
            cia.__hour = (cia.__hour == 0x91) ? 0x00 : 0x10;
          } else {
            let pm = cia.__hour & 0x80;
            cia.__hour = BCD.add(cia.__hour & 0x7F, 0x01, 2) | pm;
          }
        } else {cia.__min = BCD.add(cia.__min, 0x01, 2);}
      } else {cia.__sec = BCD.add(cia.__sec, 0x01, 2);}
    } else {cia.__10ths = BCD.add(cia.__10ths, 0x01, 2);}
  }

  if (cia.__10ths == cia.__AL10ths && cia.__sec == cia.__ALsec && cia.__min == cia.__ALmin && cia.__hour == cia.__ALhour){
    ChangeICBit(cia, 1, 2);
  }
}


class MOSCIA{
  constructor(){
    this.__regsel = 0;

    this.__CTA = 0;
    this.__CTB = 0;
    this.__IC = 0;

    this.__timerA = 0;
    this.__timerALatch = 0;
    this.__timerB = 0;
    this.__timerBLatch = 0;

    // Time Of Day
    this.__TODLatch = 0;
    this.__TODTick = 0;
    this.__10ths = 0;
    this.__sec = 0;
    this.__min = 0;
    this.__hour = 0;

    // TOD Alarm
    this.__AL10ths = 0;
    this.__ALsec = 0;
    this.__ALmin = 0;
    this.__ALhour = 0;
  }

  get RS(){return this.__regsel;}
  set RS(rs){this.__regsel = rs & 0x0F;}

  get DATA(){
    let val = 0;
    switch(this.__regsel){
      case 0x00: // Peripheral Data A
        break;
      case 0x01: // Peripheral Data B
        break;
      case 0x02: // Data Direction Reg A
        break;
      case 0x03: // Data Direction Reg B
        break;
      case 0x04: // Timer A Low
        break;
      case 0x05: // Timer A High
        break
      case 0x06: // Timer B Low
        break;
      case 0x07: // Timer B High
        break;
      case 0x08: // 10th of Sec Reg
        this.__TODLatch = 0;
        return this.__10ths; break;
      case 0x09: // Seconds Reg
        return this.__sec; break;
      case 0x0A: // Minutes Reg
        return this.__min; break;
      case 0x0B: // Hours AM/PM Reg
        this.__TODLatch = 1;
        return this.__hours; break;
      case 0x0C: // Serial Data
        break;
      case 0x0D: // Interrupt Control
        break;
      case 0x0E: // Control Timer Reg A
        break;
      case 0x0F: // Control Timer Reg B
        break;
    }
    return val;
  }

  set DATA(d){
    switch(this.__regsel){
      case 0x00: // Peripheral Data A
        break;
      case 0x01: // Peripheral Data B
        break;
      case 0x02: // Data Direction Reg A
        break;
      case 0x03: // Data Direction Reg B
        break;
      case 0x04: // Timer A Low
        break;
      case 0x05: // Timer A High
        break
      case 0x06: // Timer B Low
        break;
      case 0x07: // Timer B High
        break;
      case 0x08: // 10th of Sec Reg
        break;
      case 0x09: // Seconds Reg
        break;
      case 0x0A: // Minutes Reg
        break;
      case 0x0B: // Hours AM/PM Reg
        break;
      case 0x0C: // Serial Data
        break;
      case 0x0D: // Interrupt Control
        break;
      case 0x0E: // Control Timer Reg A
        break;
      case 0x0F: // Control Timer Reg B
        break;
    }
  }

  get CNT(){return 0;}
  set CNT(c){}

  get SP(){return 0;}
  set SP(b){}

  get TOD(){return 0;}
  set TOD(b){
    if (b <= 0 || this.__TODLatch > 0){return;}
    TODTick(this); 
  }
}


module.exports = MOSCIA;


